function W = ford(pya, pyc, cv, th, rn, wvl)
%
% FORD
%
%  Fourth-order wavefront aberrations
%
%

lambda = wvl*1e-3; % convert microns to mm
m = length(th);
L = rn(m)*(pya(m,1)*pyc(m,2)-pya(m,2)*pyc(m,1));
Ia = zeros(m,1);
Ic = Ia;
Ja = Ia;
Jc = Ia;
Jp = Ia;
for i=2:m
  Ia(i,1) = rn(i)*(pya(i,2)+cv(i)*pya(i,1));
  Ic(i,1) = rn(i)*(pyc(i,2)+cv(i)*pyc(i,1));
  Ja(i,1) = pya(i,1)*(pya(i,2)/rn(i) - pya(i-1,2)/rn(i-1));
  Jc(i,1) = pya(i,1)*(pyc(i,2)/rn(i) - pyc(i-1,2)/rn(i-1));
  Jp(i,1) = cv(i)*(1.0/rn(i)-1.0/rn(i-1));
end
W040 = -Ia.*Ia.*Ja/(8*lambda);
W131 = -Ia.*Ic.*Ja/(2*lambda);
W222 = -Ic.*Ic.*Ja/(2*lambda);
W220 = -L*L*Jp/(4*lambda) + 0.5*W222;
W311 = -Ic.*(Ic.*Jc-pyc(1:m,1).*Jp*L)/(2*lambda);
W = [W040 W131 W222 W220 W311];
%W(m,:) = sum(W); % keep all rows
W = sum(W); % keep totals
if nargout<1
    fprintf('\n%10s %10s %10s %10s %10s\n','W040','W131','W222','W220','W311');
    fprintf('%10.2f %10.2f %10.2f %10.2f %10.2f\n',W);
end
